const { h, Component } = require('preact')
const { decode } = require('reshape-preact-components')
import ArrowIcon from './arrow-icon'
import MenuIcon from './menu-icon'

export default class DocsSidenav extends Component {
  render() {
    const current = this.props.current_page.split('/').slice(1)
    const category = this.props.category
    const order = decode(this.props.order)
    const data = decode(this.props.data).map(p => {
      p.path = p.path
        .split('/')
        .slice(1)
        .join('/')
      return p
    })

    return (
      <div
        data-state={this.props._state}
        class={`g-docs-sidenav${this.state.open ? ' open' : ''}`}
      >
        <ul class="nav docs-nav">
          {this.renderNavTree(
            category,
            this.matchOrderToPageData(order, data),
            current
          )}
        </ul>
        <div class="toggle" onClick={this.toggleMenu.bind(this)}>
          {this.state.open ? (
            <span>
              <ArrowIcon /> Hide Docs Menu
            </span>
          ) : (
            <span>
              <MenuIcon /> Documentation Menu
            </span>
          )}
        </div>
      </div>
    )
  }

  // replace all terminal page nodes with page data from middleman
  matchOrderToPageData(content, pageData, stack = []) {
    // go through each item in the user-established order
    return content.map(item => {
      if (typeof item === 'string') {
        // special divider functionality
        if (item.match(/^-+$/)) return item
        // if we have a string, that's a terminal page. we match it with
        // middleman's page data and return the enhanced object
        return pageData.filter(page => {
          // break down the full path and strip the html extension
          const pageDataPath = page.path.replace(/\.html$/, '').split('/')
          // copy the stack and push the item as the file path
          const contentPath = [...stack, item]
          // match them up!
          return pageDataPath.join('-') === contentPath.join('-')
        })[0]
      } else {
        // keep track of all parent categories
        item.stack = Array.prototype.concat(stack, item.category)

        // grab the index page, as it can contain data about the top level link
        pageData.some(page => {
          const split = page.path.split('/')
          if (
            item.stack.length === split.length - 1 &&
            item.stack.every((s, i) => s === split[i]) &&
            split[split.length - 1] === 'index.html'
          ) {
            item.indexData = page
            return true
          }
        })

        // error handling for nav nesting mistakes
        if (!item.indexData) {
          throw new Error(
            `An index page is required for all categories but "${item.stack.join(
              '/'
            )}/index.html" was not found.`
          )
        }

        // otherwise, it's a nested category. if the category has content, we
        // recurse, passing in that category's content, and the matching
        // subsection of page data from middleman
        if (item.content) {
          item.content = this.matchOrderToPageData(
            item.content,
            this.filterData(pageData, item.category),
            item.stack
          )
        }
        return item
      }
    })
  }

  // recursive render for a recursive data structure!
  renderNavTree(category, content, currentPath, depth = 0) {
    return content.map(item => {
      // dividers are the only items left as strings
      if (typeof item === 'string') return <hr />

      if (item.path) {
        return (
          <li
            class={
              this.fileMatch(item.path.split('/'), currentPath) ? 'active' : ''
            }
          >
            <a
              href={`/${category}/${item.path}`}
              dangerouslySetInnerHTML={{
                __html: item.data.sidebar_title || item.data.page_title
              }}
            />
          </li>
        )
      } else {
        const title = item.indexData
          ? item.indexData.data.sidebar_title || item.indexData.data.page_title
          : item.category

        return (
          <li
            class={`${item.content ? 'dir' : ''}${
              this.categoryMatch(currentPath, item.indexData.path.split('/'))
                ? ' open active'
                : ''
            }`}
          >
            <a
              href={`/${category}/${
                item.indexData.path === 'index.html' ? '/' : item.indexData.path
              }`}
              onClick={item.content && this.toggleNav.bind(this)}
              dangerouslySetInnerHTML={{ __html: title }}
            />
            {item.content && (
              <ul class="nav">
                <li
                  class={
                    this.fileMatch(item.indexData.path.split('/'), currentPath)
                      ? 'active'
                      : ''
                  }
                >
                  <a
                    href={`/${category}/${
                      item.indexData.path === 'index.html'
                        ? '/'
                        : item.indexData.path
                    }`}
                  >
                    Overview
                  </a>
                </li>
                {this.renderNavTree(category, item.content, currentPath, depth)}
              </ul>
            )}
          </li>
        )
      }
    })
  }

  filterData(data, category) {
    return data.filter(d => d.path.split('/').indexOf(category) > -1)
  }

  categoryMatch(navItemPath, currentPath) {
    navItemPath = navItemPath.slice(0, navItemPath.length - 1)
    currentPath = currentPath.slice(0, currentPath.length - 1)
    return currentPath.reduce((result, item, i) => {
      if (item !== navItemPath[i]) result = false
      return result
    }, true)
  }

  fileMatch(navItemPath, currentPath) {
    return currentPath.reduce((result, item, i) => {
      if (item !== navItemPath[i]) result = false
      return result
    }, true)
  }

  toggleNav(e) {
    e.preventDefault()
    e.currentTarget.parentElement.classList.toggle('open')
  }

  toggleMenu() {
    this.setState({ open: !this.state.open })
  }
}
